@@ -1,4 +1,4 @@
-"spantree" <-
+`spantree` <-
     function (d, toolong = 0) 
 {
     dis <- as.dist(d)
@@ -8,7 +8,7 @@
               n = as.integer(n), val = double(n + 1),
               dad = integer(n + 1), NAOK = TRUE, PACKAGE = "vegan")
     out <- list(kid = dis$dad[2:n] + 1, dist = dis$val[2:n],
-                labels = labels, call = match.call())
+                labels = labels, n = n, call = match.call())
     class(out) <- "spantree"
     out
 }
